#
# iWeather Sensor
# ---
# Desklet code by 		Michael Favia <garbage@insitesinc.com>
# Icon sets by 			Peter Clark <pc451@yahoo.com> 
# Based on                      GoodWeather by Konrad Rieck
# Data provided by  		Weather.com - Weather XML Data Feed
#
# This sensor is being completely overhauled. Please submit improvements 
# to Michael Favia at the above email address.
#
# ---
# This software is destributed under the terms of the GNU General Public
# License. This software is free software. You may distribute and modify
# this module freely
#  
#
from sensor.Sensor import Sensor
from utils.datatypes import *

import xmltramp
import commands
import re
import os
import time
import Queue
from utils import i18n

#
# Sensor for retrieving weather information.
#
class iWeatherSensor(Sensor):

    METRIC = 'm'
    ENGLISH = 'e'

    def __init__(self):

        
        self.__queue = Queue.Queue(1)
        self.__path = os.getcwd()

        Sensor.__init__(self)

        self._set_config_type("daysToForecast", TYPE_INT, 10)
        self._set_config_type("weather_code", TYPE_STRING, "78704")
        self._set_config_type("update", TYPE_STRING, "30")
        self._set_config_type("icons", TYPE_STRING, "liquid")
        self._set_config_type("units", TYPE_STRING, self.METRIC)

        self._add_thread(self.__get_weather_thread)
        self._add_timer(0, self.__get_weather)

    #
    # Returns the configuration GUI.
    #
    def get_configurator(self):
        global _; _ = i18n.Translator("weather-sensor")
        configurator = self._new_configurator()
        configurator.set_name(_("iWeather"))

        configurator.add_title(_("Settings"))

        configurator.add_entry(_("Location Code:"), "weather_code", 
                               _("The weather.com location (USTX0057) or postal code (78704)"))

        configurator.add_entry(_("Update Interval:"), "update", 
                               _("Interval to update display in minutes"))

        configurator.add_title(_("Measurement"))
        configurator.add_option(_("Measurement unit:"), "units",
                                _("Metric or English"),
                                [(_("Metric"), self.METRIC),
                                 (_("English"), self.ENGLISH)])

        listOfDays = [(_("0"), 0),
                      (_("1"), 1), 
                      (_("2"), 2), 
                      (_("3"), 3), 
                      (_("4"), 4), 
                      (_("5"), 5),         
                      (_("6"), 6),         
                      (_("7"), 7),         
                      (_("8"), 8),         
                      (_("9"), 9),         
                      (_("10"), 10)] 

        configurator.add_title(_("Display"))

        configurator.add_option(_("Days to Forecast"), "daysToForecast",
                                _("Select the number of days to forecast"),
                                listOfDays)

        configurator.add_option(_("Icon Theme:"), "icons",
                                _("Choose an icon set"),
                                [(_("Liquid"), "liquid"),
                                 (_("UM"), "um"), 
                                 (_("weather.com"), "weather.com")])

        configurator.add_title(_("Weather XML Data Feed by weather.com"))
        
        return configurator

    #
    # Retrieves the weather data.
    #
    def __get_weather(self):

        # check the queue regularly
        if (not self.__queue.empty()):
            output = self.__queue.get()
            self._send_output(output)

        self._add_timer(500, self.__get_weather)

    #
    # Thread function for retrieving the weather data since this may take time
    # and must not block the application.
    #
    def __get_weather_thread(self):
        old_values = ("", "", "", "", "")
        current_time = int(time.time())
        # Set us back update seconds so the if statment will be true
        update = int(self._get_config("update"))
        last_update = current_time - (update * 60) - 1

        icons = self._get_config("icons")
        output = self._new_output()
        self.__queue.put(output)
        
        while (1):
            if (self._is_stopped()): break

            daysToForecast = int(self._get_config("daysToForecast"))
            weather_code = self._get_config("weather_code")
            units = self._get_config("units")
            icons = self._get_config("icons")
            update = int(self._get_config("update"))

            # Initalize the current time. 
            current_time = int(time.time())

            if ((weather_code, icons, units, update, daysToForecast) != old_values or \
                last_update + (update * 60) <= current_time):

                # At least 300/60 = 5 minutes this will run. Or so I
                # thought...  Sleep can take a few seconds to many minutes,
                # it's not accurate at all.  This is a problem for me, I'm
                # picky.  CN
                old_values = (weather_code, icons, units, update, daysToForecast)
                current_time = int(time.time())
                last_update = int(time.time())

                xmlWeather = xmltramp.load('http://xoap.weather.com/weather/local/%s?prod=xoap&par=1003832479&key=bb12936706a2d601&cc=*&dayf=%s&unit=%s' % (str(weather_code), str(daysToForecast), str(units)))
                output = self._new_output()

                try:
                    #output general info
                    output.set("forecastDays", str(daysToForecast))
                    output.set("location", str(xmlWeather.loc.dnam))
                    #Output Current Conditions
                    output.set("ccTemp", str(xmlWeather.cc.tmp) + " " + str(xmlWeather.head.ut))
                    output.set("ccFlik", "Feels: " + str(xmlWeather.cc.flik) + " " + str(xmlWeather.head.ut))
                    output.set("ccSky", "Sky: " + str(xmlWeather.cc.t))
		    output.set("ccIcon", os.path.join(self.__path, icons, "large_icons", "%s.png" % str(xmlWeather.cc.icon)))
                    output.set("ccBar", "Bar: " + str(xmlWeather.cc.bar.r) + " " + str(xmlWeather.head.up))
                    output.set("ccWind", "Wind: " + str(xmlWeather.cc.wind.s) + " " + str(xmlWeather.head.us))
                    output.set("ccHmid", "Hmid: " + str(xmlWeather.cc.hmid) + "%")
                    output.set("ccVis", "Vis: " + str(xmlWeather.cc.vis) + " " + str(xmlWeather.head.ud))
                    output.set("ccUv", "UV: " + str(xmlWeather.cc.uv.i) + " " + str(xmlWeather.cc.uv.t))
                    output.set("ccDewp", "Dew: " + str(xmlWeather.cc.dewp) + str(xmlWeather.head.ut))

                    counter = 0
                    #Output Forecast Data
                    for forecast in xmlWeather.dayf["day":]:
                        #Output allday forecasts
                        output.set("forecastTempHi[%s]" % str(counter), str(forecast.hi) + str(xmlWeather.head.ut))
                        output.set("forecastTempLow[%s]" % str(counter), str(forecast.low) + str(xmlWeather.head.ut))
                        output.set("forecastHiLow[%s]" % str(counter), str(forecast.hi) + ':' + str(forecast.low))
                        output.set("forecastSunrise[%s]" % str(counter), str(forecast.sunr)[:-3])
                        output.set("forecastSunset[%s]" % str(counter), str(forecast.suns)[:-3])
                        output.set("forecastDay[%s]" % str(counter), str(forecast("t")))
                    #end for                                                 
                        for dayparts in forecast["part":]:
                            if dayparts("p") == "d":
                                #Output daytime forecasts
                                output.set("forecastDayIcon[%s]" % str(counter), os.path.join(self.__path, icons, "small_icons", "%s.png" % str(dayparts.icon)))
                                output.set("forecastDaySky[%s]" % str(counter), str(dayparts.t))         
                                output.set("forecastDayWind[%s]" % str(counter), str(dayparts.wind.s) + str(dayparts.wind.t)[0:1])       
                                output.set("forecastDayPpcp[%s]" % str(counter), str(dayparts.ppcp) + "%")         
                                output.set("forecastDayHmid[%s]" % str(counter), str(dayparts.hmid) + "%")         
                                #Output nighttime forecasts
                            else:
                                output.set("forecastNightIcon[%s]" % str(counter), os.path.join(self.__path, icons, "small_icons", "%s.png" % str(dayparts.icon)))
                                output.set("forecastNightSky[%s]" % str(counter), str(dayparts.t))       
                                output.set("forecastNightWind[%s]" % str(counter), str(dayparts.wind.s) + str(dayparts.wind.t)[0:1])
                                output.set("forecastNightPpcp[%s]" % str(counter), str(dayparts.ppcp) + "%")       
                                output.set("forecastNightHmid[%s]" % str(counter), str(dayparts.hmid) + "%")       
                        #end for
                        #increment counter
                        counter = counter + 1
                except:
                    output.set("temperature", "N/A")
                    output.set("sky", "N/A")
                    output.set("icon", os.path.join(self.__path, icons, "large_icons", "36.png"))
                    for i in range(daysToForecast + 1):
                       output.set("forecastDayIcon" + str(i), os.path.join(self.__path, icons, "small_icons", "36.png"))
                    #end for                                                 
                self.__queue.put(output)
            #end if
            time.sleep(1)
        #end while


def new_sensor(args): return apply(iWeatherSensor, args)
